package com.health.auth.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.health.auth.feign.AuthFeignSystem;
import com.health.auth.service.UserService;
import com.health.common.constants.JwtConstants;
import com.health.common.constants.TokenConstants;
import com.health.common.domain.User;
import com.health.common.domain.req.ReqLogin;
import com.health.common.domain.resp.RespJWT;
import com.health.common.result.Result;
import com.health.common.utils.JwtUtils;
import com.health.common.utils.StringUtils;
import lombok.AllArgsConstructor;
import lombok.extern.log4j.Log4j2;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

/**
 * 用户登录业务层
 * @author Administrator
 * @date 2023年10月15日 09点48分
 */
@Service
@Log4j2
@AllArgsConstructor
public class UserServiceImpl implements UserService {

    private final AuthFeignSystem authFeignSystem;
    private final RedisTemplate<String,String> redisTemplate;


    /**
     * 登录方法--获取token
     *
     * @param reqLogin
     * @return 返回result格式的jwt包含：（token，过期时间）
     */
    @Override
    public Result<RespJWT> login(ReqLogin reqLogin) {
        if (StringUtils.isEmpty(reqLogin.getUsername())){
            return Result.error("用户名为空！");
        }
        if (StringUtils.isEmpty(reqLogin.getPassword())){
            return Result.error("密码为空！");

        }
        String username = reqLogin.getUsername();


        Result<User> userResult = authFeignSystem.byUserName(username);
        User data = userResult.getData();
        if (null == data) {
            return Result.error("先注册手机号,在登录");
        }
        HashMap<String, Object> map = new HashMap<>();
        String userkey = UUID.randomUUID().toString().replaceAll("-", "");
        map.put(JwtConstants.USER_KEY, userkey);
        map.put(JwtConstants.DETAILS_USER_ID, data.getId());
        String token = JwtUtils.createToken(map);
        redisTemplate.opsForValue().set(
                TokenConstants.LOGIN_TOKEN_KEY + userkey,
                JSONObject.toJSONString(data),
                TokenConstants.EXPIRATION,
                TimeUnit.MINUTES
        );
        RespJWT respJWT = new RespJWT();
        respJWT.setToken(token);
        respJWT.setEidTime("720MIN");

        return Result.success(respJWT);

    }

    /**
     * 每次都对请求做权限判断
     *
     * @param request
     * @return 返回result格式的User对象包含：（用户信息）
     */
    @Override
    public Result<User> userinfo(HttpServletRequest request) {
        return null;
    }
}
